% Reporting bugs in darcs.  See also impossible.h.

\begin{code}
module Darcs.Bug ( _bug, bugDoc,
                  _bugLoc, _impossible, _fromJust, _withBugLoc,
                 ) where

import Printer ( Doc, errorDoc, text, ($$) )
\end{code}

\begin{code}
_bug :: BugStuff -> String -> a
_bug bs s = error $ "bug in darcs!\n" ++ s ++ " at " ++ _bugLoc bs ++
                    "\nPlease report this to bugs@darcs.net," ++
                    "\nIf possible include the output of 'darcs --exact-version'.\n\n"

bugDoc :: Doc -> a
bugDoc s = errorDoc $ text "bug in darcs!"
                   $$ s
                   $$ text "Please report this to bugs@darcs.net"
                   $$ text "If possible include the output of 'darcs --exact-version'."
\end{code}

\begin{code}
type BugStuff = (String, Int, String, String)

_bugLoc :: BugStuff -> String
_bugLoc (file, line, date, time) = file++":"++show line++" compiled "++time++" "++date

_impossible :: BugStuff -> a
_impossible bs = _bug bs $ "Impossible case"++_bugLoc bs

_fromJust :: BugStuff -> Maybe a -> a
_fromJust bs mx =
  case mx of Nothing -> _bug bs $ "fromJust error"++_bugLoc bs
             Just x  -> x

_withBugLoc :: BugStuff -> IO a -> IO a
_withBugLoc bs job = job `catch` \err -> error $ _bugLoc bs++"\n"++show err
\end{code}
