{-# OPTIONS -fffi #-}
module Libwww ( copyUrl, waitNextUrl, lastUrl )
where

import System.IO
#ifdef HAVE_LIBWWW
import Foreign.C.Types ( CInt )
import Foreign.C.String ( withCString, peekCString, CString )
import System.Environment ( getEnv )
import Autoconf ( darcs_version )
import Darcs.Progress ( debugMessage )
#endif
import Curl ( Cachable(..) )

copyUrl :: String -> String -> Cachable -> IO String
#ifdef HAVE_LIBWWW
copyUrl u f cache =
  withCString darcs_version $ \vstr ->
  withCString u $ \ustr ->
  withCString f $ \fstr -> do
  ppwd <- getProxyUserPwd
  withCString ppwd $ \pstr -> do
  debugMessage ("Libwww.copyUrl ("++u++"\n"++
                "              -> "++f++")")
  err <- libwww_request_url vstr pstr ustr fstr (cachableToInt cache)
  if err /= 0
     then return $ "Failed to start download URL " ++ u ++ " : " ++ libwwwError err
     else do debugMessage ("Libwww.copyUrl succeeded")
             return ""
#else
copyUrl _ _ _ = fail "There is no libwww!"
#endif

waitNextUrl :: IO String
#ifdef HAVE_LIBWWW
waitNextUrl = do err <- libwww_wait_next_url
                 if (err /= 0)
                    then do u <- lastUrl
                            return $ "Failed to download URL " ++ u ++ " : " ++ libwwwError err
                    else do debugMessage ("Libwww.waitNextUrl succeeded")
                            return ""
#else
waitNextUrl = fail "There is no libwww!"
#endif

lastUrl :: IO String
#ifdef HAVE_LIBWWW
lastUrl = libwww_last_url >>= peekCString
#else
lastUrl = fail "There is no libwww!"
#endif

#ifdef HAVE_LIBWWW
libwwwError :: CInt -> String
libwwwError 1000 = "HTRequest_new()"
libwwwError 1001 = "HTRequest_addAfter() failed"
libwwwError 1002 = "HTRequest_addCacheControl() failed"
libwwwError 1003 = "HTLoadToChunk() failed"
libwwwError 1004 = "fopen() failed"
libwwwError 1005 = "fwrite() failed"
libwwwError err  = "libwww error code: " ++ show err

cachableToInt :: Cachable -> CInt
cachableToInt Cachable = -1
cachableToInt Uncachable = 0
cachableToInt (MaxAge n) = n

foreign import ccall "hslibwww.h libwww_request_url"
  libwww_request_url :: CString -> CString -> CString -> CString -> CInt -> IO CInt

foreign import ccall "hslibwww.h libwww_wait_next_url"
  libwww_wait_next_url :: IO CInt

foreign import ccall "hslibwww.h libwww_last_url"
  libwww_last_url :: IO CString

getProxyUserPwd :: IO String
getProxyUserPwd = do
 getEnv "DARCS_PROXYUSERPWD" `catch` (\_ -> return "")
#endif
