module URL ( copyUrl, copyUrlFirst, waitNextUrl, waitUrl,
             Cachable(Cachable, Uncachable, MaxAge)
           ) where

import Data.IORef
import Data.Set ( Set )
import qualified Data.Set as Set
import System.IO.Unsafe ( unsafePerformIO )
import Monad ( when )
import qualified Libwww as WWW ( copyUrl, waitNextUrl, lastUrl )
import Curl ( Cachable(..) )

data UrlState = UrlState { inProgress :: Set String,
                           waitToStart :: [(String, String, Cachable)],
                           pipeLength :: Int
                         }

data Priority = High | Low

maxPipeLength :: Int
maxPipeLength = 5

urlState :: IORef UrlState
urlState = unsafePerformIO $ newIORef (UrlState Set.empty [] 0)

copyUrlWithPriority :: Priority -> String -> String -> Cachable -> IO ()
copyUrlWithPriority prio url file c = do
  st <- readIORef urlState
  let p = inProgress st
      w = waitToStart st
      new_w = case prio of
                High -> (url, file, c):w
                Low  -> w ++ [(url, file, c)]
      new_st = st { inProgress = Set.insert url p,
                    waitToStart = new_w
                  }
  writeIORef urlState new_st
  checkWaitToStart

copyUrlFirst :: String -> String -> Cachable -> IO ()
copyUrlFirst = copyUrlWithPriority High

copyUrl :: String -> String -> Cachable -> IO ()
copyUrl = copyUrlWithPriority Low

waitNextUrl :: IO String
waitNextUrl = do st <- readIORef urlState
                 let l = pipeLength st
                 if l > 0
                    then do err <- WWW.waitNextUrl
                            url <- WWW.lastUrl
                            let p = inProgress st
                                new_st = st { inProgress = Set.delete url p,
                                              pipeLength = l - 1
                                            }
                            writeIORef urlState new_st
                            checkWaitToStart
                            when (not $ null err) $ fail err
                            return url
                    else return ""

waitUrl :: String -> IO ()
waitUrl u = do st <- readIORef urlState
               when (u `Set.member` inProgress st) waitUrl'
    where waitUrl' = do url <- waitNextUrl
                        when (u /= url) waitUrl'

checkWaitToStart :: IO ()
checkWaitToStart = do st <- readIORef urlState
                      let l = pipeLength st
                      when (l < maxPipeLength) $
                           do let w = waitToStart st
                              case w of
                                (u,f,c):rest -> do
                                       let new_st = st { waitToStart = rest,
                                                         pipeLength = l + 1
                                                       }
                                       writeIORef urlState new_st
                                       err <- WWW.copyUrl u f c
                                       when (not $ null err) $ fail err
                                       checkWaitToStart
                                _ -> return ()
