#!/usr/bin/env perl

# A start on testing amend-record. Please add more tests for amend here!

use lib 'lib/perl';
use Test::More 'no_plan';
use strict;
use Test::Darcs;
use Shell::Command;
use File::Slurp;


cleanup 'temp1';
mkpath 'temp1';
chdir 'temp1';

like( darcs('init'), qr/^$/i, 'initialized repo');
$ENV{EMAIL} = 'Tester';

# avoid possible problems with patch name collisions happening in the same second. 
append_file('_darcs/prefs/defaults',"ALL ignore-times\n");

# Plain amend-record
touch 'foo';
darcs "add foo";
like( darcs("record -a -m add_foo"), qr/finished recording/i, 'added patch');
append_file('foo',"\nanother line");

like( echo_to_darcs("y","amend-record -a foo"), qr/amending changes/i, 'amend-record -a');
is($?,0, " return code == 0");

{
  my $changes = darcs "changes -v";
  like( $changes, qr/another line/, 'change amended properly');
}

# Special case: patch is empty after amend
cp "foo","foo.old";
append_file('foo',"\nanother line");

like( darcs("record -a -m add_line foo"), qr/finished recording/i, 'record');
mv "foo.old","foo";
like( echo_to_darcs("y","amend -a foo"), qr/amending changes/i, 'amend makes empty patch');
is($?,0, "  return code == 0");

# Amend --author, -m, etc
append_file('foo',"\nanother line");
like( echo_to_darcs("y","amend -a -m new_name foo"), qr/amending changes/i, 'amend');
like( darcs("changes --last=1"), qr/new_name/, 'amend can change patch name' );

append_file('foo',"\nanother line");

like( echo_to_darcs("y","amend -a -m new_name -A new_author foo"), qr/amending changes/i, 'amend');
like( darcs("changes --last=1"), qr/new_author/, 'amend can change patch author' );

# Cleanup
chdir '../';
rm_rf 'temp1';
