#!/usr/bin/env perl

# Tests for the doppleganger conflict bug.

# For Zooko, with love
# Also, for issue81.

# A note about all the 'sleep' statements: When I had less of them in the code to slow
# it down, some tests would fail intermittently, because darcs would not detect changes
# had happened. It would be good to remove any that are possible to speed up the test,
# but test running the script 3 to 5 times if you do that, to confirm the intermittent 
# failure is gone. - MarkStosberg

use lib 'lib/perl';

use Test::More 'no_plan';
use Test::Darcs;
use Shell::Command;
use File::Slurp;
use strict;
use warnings;

# Set the author.
$ENV{EMAIL} = 'tester';

ok (dopplegangers_conflict('old-fashioned-inventory'), "default repo format views doppleganger as a conflict");
ok (dopplegangers_conflict('hashed'),        "hashed  repo format views doppleganger as a conflict");
ok ((not dopplegangers_conflict('darcs-2')), "with darcs-2 format, doppleganger's don't conflict! ");

sub dopplegangers_conflict {
    my $arg_to_init = shift;

    cleanup  'tmp_dopple';
    cleanup  'tmp_ganger';
    mkpath 'tmp_dopple';
    chdir  'tmp_dopple';
    darcs 'init --'.$arg_to_init;

    # Map args to formats
    my %fmts = (
        'old-fashioned-inventory' => 'darcs-1.0',
        'darcs-2'                 => 'darcs-2-experimental',
        'hashed'                  => 'hashed',
    );

    like((darcs 'show repo'), qr/$fmts{$arg_to_init}/,
         "reality check: $fmts{$arg_to_init} is repo format of tmp_doppel");

    touch 'a.txt';
    darcs "add a.txt";
    my $out = darcs "record -A base -am 'adding a.txt'";
    like($out, qr/finished/msxi, "$arg_to_init: recorded patch for adding file.");

    chdir '../';

    sleep 1;

    $out = darcs 'get --'.$arg_to_init.' tmp_dopple tmp_ganger';
    #diag $out;

    #like((darcs 'show repo --repodir=tmp_ganger'), qr/$fmts{$arg_to_init}/,
    #     "reality check: $fmts{$arg_to_init} is repo format for tmp_ganger");

    for my $repo (qw(dopple ganger)) {
        ok((chdir  "tmp_$repo"), "chdir'ed");
        `echo "text which appears in both places at once" > a.txt`;
        sleep 1; # avoid identical patch names. 
        my $out = darcs "record -A $repo -am 'recording an identical change'";
        like($out, qr/finished/msxi, "$arg_to_init: recorded patch in $repo");
#        diag "in loop: $out";
        chdir '../';
    };

    # Now that the conflict has been set up, try pull one patch from the other.
    chdir 'tmp_ganger';

    my $pull_out = darcs 'pull -a ../tmp_dopple';
    my $has_a_conflict_p = ($pull_out =~ m/conflict/msxi);
#    diag "result: $pull_out";

    ####

    chdir '../';
    cleanup 'tmp_ganger';
    ok((!-d 'tmp_ganger'), 'tmp_ganger directory was deleted');
    cleanup 'tmp_dopple';
    ok((!-d 'tmp_dopple'), 'tmp_dopple directory was deleted');

#    diag "about to to return: $has_a_conflict_p";

    return $has_a_conflict_p;
}


