#!/usr/bin/env perl

# A test for darcs resolve detecting a conflict, inspired by bug #152 in RT

use Test::More 'no_plan';
use lib 'lib/perl';
use Test::Darcs;
use Shell::Command;
use strict;
use warnings;

cleanup 'tmp1';
mkpath 'tmp1';
chdir 'tmp1' || die;
darcs 'init';

open(T1,">one.txt");
print T1 "from tmp1";
close(T1);

darcs(q( add one.txt));
darcs(q( rec -A bar -am "add one.txt"));

open(T1,">>one.txt");
print T1 "\n";
close(T1);

open(PREF,">_darcs/prefs/defaults");
print PREF "apply allow-conflicts\n";
close(PREF);

chdir "../";
cleanup 'tmp2';
darcs 'get tmp1 tmp2';

chdir "tmp2/" || die "couldn't chdir: $!";

like(darcs(q(query manifest)), qr/one.txt/, "reality check: manifest");

open(T2,">>one.txt") || die "failed to open one.txt: $!";
print T2 "in tmp2\n" || die "failed to print: $!";
close(T2) || die "failed to close: $!";

like(darcs(q( w -s)), qr/M/, "reality check: darcs w -s");

my $out = darcs 'rec -A bar -am "add extra line"';

like(darcs(q( push -a)) ,qr/conflicts/i,'expect conflicts when pushing');  
chdir '../tmp1';

TODO: {
    local $TODO = 'waiting on code to fix this';
    unlike(darcs(q(resolve)),qr/no conflicts/i, "after a conflict, darcs resolve should report a conflict");
}

chdir '../';
for my $dir (qw/tmp1 tmp2/) {
    ok(-d $dir, "$dir exists here");
    cleanup "$dir";
    ok((!-d $dir), "$dir directory was deleted");
}





